#include<math.h>
#include<stdlib.h>
#include<stdio.h>

#include "../../include/VandP.h"
#include"SLHAplus.h"
#include"../ntools/include/vegas.h"
#include"dynamic_cs.h"
#include "vp.h"

extern double simpson(double (*func)(double), double a, double b, double eps);
extern double alpha_2(double Q);
/*===========================================================*/

double (*sqme)(int nsub,double GG, REAL *pvect, int * err_code)=NULL;

static int  nsub_stat;

static REAL*Q=NULL;

double  decayPcm(double am0,  double  am1,  double  am2)
{
  double  summ, diffm;
  summ = am1 + am2;
  diffm = am1 - am2;
  if(am0<summ) return 0;
  return sqrt((am0-summ)*(am0+ summ)*(am0-diffm)*(am0+diffm))/(am0*2);
}

static void setQforParticle(char*pname)
{
  char *nm;
  REAL*ma;  
  int n,i,cdim;
  int pdg;
  if(!Q) return;
 
  n=pTabPos(pname);
  if(!n){printf("Wrong particle name '%s'\n",pname); return ;}
  nm=ModelPrtcls[abs(n)-1].mass;
  if(nm[0]=='0') return ; else ma=varAddress(nm);

  cdim=abs(ModelPrtcls[abs(n)-1].cdim);
  pdg=abs(ModelPrtcls[abs(n)-1].NPDG);
  
  if(cdim==1){ calcMainFunc(); *Q=fabs(*ma);  calcMainFunc(); return;}
  switch(pdg)
  { case 1:case 2:case 3: *Q=1; return;
    case 4: *Q=1.5; break;
    case 5: *Q=5;   break;
    case 6: *Q=175; break;
  }
  calcMainFunc();
  for(i=0;i<10;i++) 
  { 
    if( fabs(*Q-fabs(*ma)) < 1E-2*(*Q)) break;
    *Q=fabs(*ma);
    calcMainFunc();
  }
} 



int ForceUG=0;



int  procInfo1(numout*cc, int *nsub, int * nin, int *nout)
{
  if(nin) *nin=cc->interface->nin;
  if(nout)*nout=cc->interface->nout;
  if(nsub)*nsub=cc->interface->nprc;
  return 0;
}

int procInfo2(numout*cc,int nsub,char**name,REAL*mass)
{
  int i;
  int ntot=cc->interface->nin+cc->interface->nout;
    
  if(nsub<1 || nsub> cc->interface->nprc) return 2;

  if(name)for(i=0;i<ntot ;i++) 
  name[i]=(cc->interface->pinf)(nsub,i+1,NULL,NULL);

  if(mass)
  {  
    for(i=1;i<=cc->interface->nvar;i++) if(cc->link[i]) cc->interface->va[i]=*(cc->link[i]);  
    if(cc->interface->calcFunc()>0) { printf("cannot calculate constr\n"); return 4;}
    for(i=0;i<ntot ;i++) cc->interface->pinf(nsub,i+1,mass+i,NULL);     
  }
  return 0;
}





                    
int pname2lib(char*pname, char * libname)
{
  int n;
  n=pTabPos(pname);
  if(!n) {printf("Wrong particle name '''%s'''\n",pname); libname[0]=0; return 1;}
  if(n>0) sprintf(libname,"p%d",n); else sprintf(libname,"a%d",-n);
  return 0;
}


static int decodeProcess(char *txt,int*inList,int*outList)
{ char name[20];
   char *ch_,*ch;
   int i,p;   
   ch_=strstr(txt,"->");
   if(!ch_) { inList[0]=0; ch=txt;}
   else
   { 
     for(p=0,ch=txt;; )
     { sscanf(ch," %[^,]",name);
       ch_=strstr(name,"->");
       if(ch_) *ch_=0;
       for(i=strlen(name)-1; i>=0 && name[i]==' '; i--) name[i]=0;
       inList[p]=pTabPos(name);
       if(!inList[p]) return -(p+1);
       p++;
       if(ch_) break;     
       ch=strchr(ch,',');
       if(!ch) break; else ch++;
     }  
     inList[p]=0; 
     ch=strstr(txt,"->")+2; 
   }  

   for(p=0;ch; )
   { sscanf(ch," %[^,]",name);
     for(i=strlen(name)-1;i>=0 && name[i]==' '; i--) name[i]=0;
     outList[p]=pTabPos(name);
     if(!outList[p]) return p+1;
     p++;    
     ch=strchr(ch,',');
     if(!ch) break;
     ch++;
   }
   outList[p]=0;
   return 0;
}


 
void massFilter(double M, txtList * List)
{
  txtList lold=*List, lnew=NULL,lnext;

  while(lold)
  {  double Msum=0,dM;
     char *ch;
     ch=strstr(lold->txt,"->")+2;
     for( ; ch; ch=strchr(ch,','))
     { char buff[10];
       int n;
       char *nm;
        
       ch++;
       sscanf(ch,"%[^,]",buff); 
       
       n=pTabPos(buff);
       nm=ModelPrtcls[abs(n)-1].mass;
       if(nm[0]=='0')   
       {  dM=0;
          switch(abs(ModelPrtcls[abs(n)-1].NPDG))
          {
            case 1: case 2: dM=0.07; break;
            case 3: dM=0.3; break;
            case 4: dM=1.5; break;
            case 5: dM=5. ; break;
          }
       }  
       else  dM = fabs(*(varAddress(nm))); 
       Msum+=dM;
     } 
     lnext=lold->next;
     if(M>Msum) {lold->next=lnew; lnew=lold;}
     else {free(lold->txt); free(lold);}
     lold=lnext;
 } 
 *List=lnew;
}

void gammaGluFilter(txtList * List)
{
  txtList lold=*List, lnew=NULL,lnext;

  while(lold)
  {  int del=0,code;
     char *ch;
     ch=strstr(lold->txt,"->")+2;
     for( ; !del && ch; ch=strchr(ch,','))
     { char buff[10];
       ch++;
       sscanf(ch,"%[^,]",buff); 
       code=pNum(buff);
       if(code==22 || code ==21) { del=1;}
     } 
     lnext=lold->next;
     if(del) {free(lold->txt); free(lold);}
     else    {lold->next=lnew; lnew=lold;}
  
     lold=lnext;
 } 
 *List=lnew;
}


int process2Lib(char * process,char * lib)
{ 
  char * ch, *ch1;
  char bufflib[20];
  char *process_;
  int err=0;
  process_=malloc(strlen(process)+1);
  strcpy(process_,process);
    
  ch= strstr(process_,"->");
  ch[0]=' '; ch[1]=','; ch=ch+2; for(;*ch==' ';ch++);

  lib[0]=0;
  ch1=strtok(process_," ,"); 
  for(;ch1 && err==0;)
  { if(ch1==ch) strcat(lib,"_");
    err=pname2lib(ch1,bufflib);
    strcat(lib,bufflib);
    ch1=strtok(NULL," ,");
  }
  free(process_);
  return err;
}

void process2Mass(char * process,double * mass)
{ 
  char * ch, *ch1;
  char *process_;
  int i;  

  process_=malloc(strlen(process)+1);
  strcpy(process_,process);
    
  ch= strstr(process_,"->");
  ch[0]=' '; ch[1]=','; ch=ch+2; for(;*ch==' ';ch++);

  ch1=strtok(process_," ,"); 
  for(i=0;ch1;i++,ch1=strtok(NULL," ,"))
  { /*if(ch1==ch) strcat(lib,"_");*/
    mass[i]=pMass(ch1);
  }
  free(process_);
}


/*===========================================================*/
static REAL pmass[5];

/* ======================decayTable ===================*/

 decayTableStr* decayTable=NULL;
 
 void cleanDecayTable(void)
 { int i,j;
   if(decayTable)
   {  
     for(i=0;i<nModelParticles;i++)
     {  
        decayTable[i].width=0;
        for(j=0;j<2;j++)  
        if(decayTable[i].pdList[j]) cleanTxtList(decayTable[i].pdList[j]);
     }
   }  
   else  decayTable=malloc(nModelParticles*sizeof(decayTableStr));
   for(i=0;i<nModelParticles;i++)
   { for(j=0;j<2;j++) decayTable[i].pdList[j]=NULL;
     decayTable[i].width=0;
   }
 }


/*======================  1->2 decay ==================*/

double pWidth2(numout * cc, int nsub)
{
  REAL pvect[12];
  double width=0.;
  REAL m1,m2,m3; 
  int i,ntot,nin,nout;
  double GG;
  procInfo1(cc,&ntot,&nin,&nout);
  if(nsub<1 ||  nsub>ntot|| nin!=1||nout !=2)  return 0;
       
  for(i=1;i<=cc->interface->nvar;i++) if(cc->link[i]) cc->interface->va[i]=*(cc->link[i]);
  
  if( cc->interface->calcFunc()>0 ) { return -1;}
  cc->interface->pinf(nsub,1,&m1,NULL);
  cc->interface->pinf(nsub,2,&m2,NULL); 
  cc->interface->pinf(nsub,3,&m3,NULL);
  if(cc->SC) GG=*(cc->SC); else GG=sqrt(4*M_PI*alpha_2(m1));
  
  if(m1 >m2 + m3)
  {   int i,err_code; 
      double md=m2-m3;
      double ms=m2+m3;
      double pRestOut=sqrt((m1*m1 - ms*ms)*(m1*m1-md*md))/(2*m1);
      double totcoef= pRestOut/(8. * M_PI * m1*m1);
           
      for(i=1;i<12;i++) pvect[i]=0;
      pvect[0]=m1;
      pvect[7]=pRestOut;
      pvect[4]=sqrt(pRestOut*pRestOut+m2*m2);
      pvect[11]=-pRestOut;
      pvect[8]=sqrt(pRestOut*pRestOut+m3*m3);
      width = totcoef * (cc->interface->sqme)(nsub,GG,pvect,&err_code);
  }
  return width;
}

 
double decay2Info(char * pname, FILE* f)
{ int i,j,ntot;
  numout * cc;
  double wtot;
  char pname2[20],process[20],plib[20];
  char * dname[8];

  for(i=0,j=0;pname[i];i++)
  if(pname[i]!=' ') pname2[j++]=pname[i];
  pname2[j]=0;
  strcpy(plib,"2width_");
  pname2lib(pname2,plib+7);
  sprintf(process,"%s->2*x",pname2);
  cc=getMEcode(0,ForceUG,process,NULL,"",plib);
  if(!cc) return -1; 
  procInfo1(cc,&ntot,NULL,NULL);
  if(f) fprintf(f,"\n Partial width for %s->2x decays in GeV\n",pname2); 
  for(wtot=0,i=1;i<=ntot;i++)
  { double w;
    procInfo2(cc,i,dname,NULL);
    w=pWidth2(cc,i);
    if(w!=0)
    { wtot+=w;
      if(f) fprintf(f,"%3.3s %3.3s  %.2E\n",dname[1],dname[2],w); 
    }
  }
  if(f) fprintf(f," Total width %.2E GeV\n",wtot);
  return  wtot;
}

static double decay22List(char * pname, txtList *LL)
{ int i,j,ntot;
  numout * cc;
  double wtot,w;
  char pname2[20],process[20],plib[20];
  char * dname[8];
  txtList L=NULL,l;
  char buff[100];

  for(i=0,j=0;pname[i];i++)
  if(pname[i]!=' ') pname2[j++]=pname[i];
  pname2[j]=0;
  strcpy(plib,"2width_");
  pname2lib(pname2,plib+7);
  sprintf(process,"%s->2*x",pname2);
  cc=getMEcode(0,ForceUG,process,NULL,"",plib);
  if(!cc) { if(LL) *LL=NULL; return -1;} 
  procInfo1(cc,&ntot,NULL,NULL);
  for(wtot=0,i=1;i<=ntot;i++)
  { 
    procInfo2(cc,i,dname,NULL);
    w=pWidth2(cc,i);
    if(w!=0)
    {  if(LL)
       { l=malloc(sizeof(txtListStr));
         l->next=L;
         L=l;
         sprintf(buff,"%E  %s -> %s,%s",w,pname2,dname[1],dname[2]);
         l->txt=malloc(20+strlen(buff));
         strcpy(l->txt,buff);
       } 
       wtot+=w; 
    }
  }

  if(LL)
  {  for(l=L;l;l=l->next)
     { 
       sscanf(l->txt,"%lf %[^\n]",&w,buff);
       sprintf(l->txt,"%E %s",w/wtot,buff);
     }   
    *LL=L;
  }
  return  wtot;
}


/*==================  1->3 decay ===================*/

static double _x_;

static double kimematic_1_3(REAL *pmass, int kin, double xm2, double xcos, REAL * P)
{ 
  double factor,pout,mQmin,mQmax,mQ,m12,chY,shY,xsin;
  double p4,p8,p5,p9;
  double m0=pmass[0],m1=pmass[1],m2=pmass[2],m3=pmass[3];
  int i;
  
  P[0]=m0; P[1]=P[2]=0;
  for(i=0;i<4;i++) P[3+i*4]=0;
  
  mQmin=(m1+m2)*(m1+m2);
  mQmax=(m0-m3)*(m0-m3);
  mQ=mQmin*(1-xm2)+mQmax*xm2;
  m12=sqrt(mQ);
   
  factor=(mQmax-mQmin)/(128*M_PI*M_PI*M_PI*m0*m0*m12);
  
  pout=decayPcm(m0,m12,m3);
  P[12]=sqrt(pout*pout+m3*m3); P[13]=-pout; P[14]=0;

  factor*=pout;  
  
  chY=sqrt(1+pout*pout/mQ);
  shY=sqrt(pout*pout/mQ);
  
  pout=decayPcm(m12,m1,m2);
  factor*=pout;
  xsin=sqrt(1-xcos*xcos);
  p4=sqrt(m1*m1+pout*pout);    p8=sqrt(m2*m2+pout*pout);
  p5=xcos*pout;                p9=-p5;
  P[6]=xsin*pout;              P[10]=-P[6];
  
  P[4]=chY*p4 + shY*p5;    P[8]=chY*p8 + shY*p9;
  P[5]=shY*p4 + chY*p5;    P[9]=shY*p8 + chY*p9;
    
  return factor;
}

static double GG;

static double dWidthdCos(double xcos)
{
  double factor;
  REAL P[16];
  int err_code;
  factor=kimematic_1_3(pmass,1,_x_,xcos, P); 
  return factor*(*sqme)(nsub_stat,GG,P,&err_code);
}

static double dWidthdM(double x)
{ _x_=x; return simpson(dWidthdCos,-1.,1.,1.E-4); }



static double width13(numout * cc, int nsub, int * err) 
{
  int i;

  for(i=1;i<=cc->interface->nvar;i++) if(cc->link[i]) cc->interface->va[i]=*(cc->link[i]);

  if( cc->interface->calcFunc()>0 ) {*err=4; return 0;}

  for(i=0;i<4;i++) cc->interface->pinf(nsub,1+i,pmass+i,NULL);
  if(cc->SC ) GG=*(cc->SC); else  GG=sqrt(4*M_PI*alpha_2(pmass[0]));
  *err=0;

  *(cc->interface->gtwidth)=0;
  *(cc->interface->twidth)=0;
  *(cc->interface->gswidth)=0;
  
  
  sqme=cc->interface->sqme;
  nsub_stat=nsub; 
  return simpson(dWidthdM,0.000001,0.99999,1.E-2);
}


static txtList conBrList(txtList BrList)
{ txtList out=NULL;
  char buff[100];
  double br;
  int inCode[10], outCode[10],i;
  for(;BrList;BrList=BrList->next)
  { txtList new=malloc(sizeof(txtListStr));
    new->next=out;out=new;
    sscanf(BrList->txt,"%lf %[^\n]",&br,buff); 
    decodeProcess(buff,inCode,outCode);
    if(inCode[0]>0) sprintf(buff,"%E  %s -> ",br,ModelPrtcls[inCode[0]-1].aname);   
    else            sprintf(buff,"%E  %s -> ",br,ModelPrtcls[-inCode[0]-1].name);
    for(i=0;outCode[i];i++)
    { if(i) strcat(buff,",");
      if(outCode[i]>0) strcat(buff,ModelPrtcls[outCode[i]-1].aname);   
      else             strcat(buff,ModelPrtcls[-outCode[i]-1].name);  
    }
    new->txt=malloc(strlen(buff)+1);
    strcpy(new->txt,buff);
  }
  return out;  
}



static double kimematic_1_4(REAL *pmass, double xm1, double xm2, double xcos1, double xcos2,double fi2,  REAL * P)
{ 
  double factor,M1,M2,Pcm,p1cm,p2cm,chY,shY,xsin;
  double p0,p3;
  int i,j;

  factor= 1./(pow(2*M_PI,8)*2*pmass[0]*16);
  
  M1= pmass[1]+pmass[2]+ xm1*(pmass[0]-pmass[1]-pmass[2]-pmass[3]-pmass[4]);
  M2= pmass[3]+pmass[4]+ xm2*(pmass[0]-M1-pmass[3]-pmass[4]); 

  factor*=(pmass[0]-pmass[1]-pmass[2]-pmass[3]-pmass[4])*(pmass[0]-M1-pmass[3]-pmass[4]);
  Pcm=decayPcm(pmass[0],M1,M2);            factor*=4*M_PI*Pcm/pmass[0];
  p1cm=decayPcm(M1,pmass[1],pmass[2]);     
  p2cm=decayPcm(M2,pmass[3],pmass[4]);     factor*=2*M_PI*p1cm*p2cm;
  
  
  P[0]=pmass[0]; P[1]=P[2]=P[3]=0;
  
  P[4+0]=sqrt(pmass[1]*pmass[1]+p1cm*p1cm);       P[8+0]=sqrt(pmass[2]*pmass[2]+p1cm*p1cm);
  P[4+1]=0;   
  P[4+2]=p1cm*sqrt(1-xcos1*xcos1);
  P[4+3]=p1cm*xcos1;
  
          
  P[12+0]=sqrt(pmass[3]*pmass[3]+p2cm*p2cm);       P[16+0]=sqrt(pmass[4]*pmass[4]+p2cm*p2cm);
  P[12+1]=sin(fi2)*p2cm*sqrt(1-xcos2*xcos2);
  P[12+2]=cos(fi2)*p2cm*sqrt(1-xcos2*xcos2); 
  P[12+3]=p2cm*xcos2;  
  
  for(i=1;i<=2;i++) for(j=1;j<=3;j++) P[i*8+j]=-P[i*8-4+j];
  shY=Pcm/M1;
  chY=sqrt(1+shY*shY);
  for(i=1;i<3;i++)
  { double  p0=P[4*i];
    double  p3=P[4*i+3];
    P[4*i]=chY*p0+shY*p3;
    P[4*i+3]=shY*p0+chY*p3;
  }

  shY=-Pcm/M2;            
  chY=sqrt(1+shY*shY);  
  for(i=3;i<5;i++)
  { double  p0=P[4*i];
    double  p3=P[4*i+3];
    P[4*i]=chY*p0+shY*p3;
    P[4*i+3]=shY*p0+chY*p3;
  }

//printf("Energy conservation\n");
for(i=0;i<4;i++)
{ double sum=P[0+i]-P[4+i]-P[8+i]-P[12+i]-P[16+i];
  if(fabs(sum/P[0]) > 1.E-4)
  { printf("No Energy conservation %E i=%d  \n",sum/P[0],i);
    exit(22);
  }  
}    



  for(i=0;i<5;i++)
  { double m;
    m=sqrt(fabs(P[4*i]*P[4*i]-P[4*i+1]*P[4*i+1]-P[4*i+2]*P[4*i+2]-P[4*i+3]*P[4*i+3]));
    if(fabs(m-pmass[i])>pmass[0]*1.E-5) { printf("wrong mass %d (%E != %E) \n",i,m,pmass[i]); exit(33);}
  }


  return factor;
}

static double wInt14(double *x, double w)
{
   REAL pvect[20];
   int err_code;
   double res;
   res=kimematic_1_4(pmass,x[0], x[1], 2*(x[2]-0.5),2*(x[3]-0.5),2*M_PI*x[4],pvect);   
   res*= (*sqme)(1,GG, pvect, &err_code);
   return res*8*M_PI;
}

static double width14(numout * cc, int * err) 
{
  int i;
  vegasGrid * vegPtr;
  long ncall0=10000;                     /* number of integrand calls */
  double alph=1.5;                       /* rate of grid improvement  */
  double ti;                             /* integral estimation */
  double tsi;                            /* standard deviation */
                          

  for(i=1;i<=cc->interface->nvar;i++) if(cc->link[i]) cc->interface->va[i]=*(cc->link[i]);

  if( cc->interface->calcFunc()>0 ) {*err=4; return 0;}
    
  for(i=0;i<5;i++) cc->interface->pinf(1,1+i,pmass+i,NULL);  
  if(cc->SC ) GG=*(cc->SC); else  GG=sqrt(4*M_PI*alpha_2(pmass[0]));
  *err=0;

  *(cc->interface->gtwidth)=0;
  *(cc->interface->twidth)=0;
  *(cc->interface->gswidth)=0;  
  sqme=cc->interface->sqme;
  
  vegPtr=vegas_init(5,50);
  
  vegas_int(vegPtr,ncall0,alph,wInt14,&ti,&tsi); 
  vegas_int(vegPtr,ncall0,alph,wInt14,&ti,&tsi);
  vegas_finish(vegPtr);
   
  return ti;
}



double pWidth(char *name, txtList * LL)
{
  txtList L,l,Lout;
  char libName[100];
  double sum=0,width;
  int i,i0,j,j0,nout;
  REAL Qstat;
  Q=NULL;
  
  
  for(i=0;i<nModelParticles;i++)
  { char *pnames[2]={ModelPrtcls[i].name,ModelPrtcls[i].aname};
    for(j=0;j<2;j++)
    if(strcmp(name,pnames[j])==0) 
    { if(decayTable[i].pdList[j])
      { 
        if(LL) *LL=decayTable[i].pdList[j];
        return decayTable[i].width;
      } else break;
    } if(j!=2) break;    
  }    

  i0=i,j0=j;
  if(i0==nModelParticles)
  { printf("%s out of model particles\n",name);
    if(LL) *LL=NULL;
    return 0;
  }  

  {  int pdg,pdg0,Len,decay[10];
     double br;
     pdg0=ModelPrtcls[i0].NPDG;
     if(j0) pdg0=-pdg0;
     for(i=1; allDecays(i,0,&pdg,&Len,decay,&width,&br) ;i++)
     {
        if(abs(pdg)==abs(pdg0))
        {  txtListStr*l,*L=NULL;
           l=malloc(sizeof(txtListStr));
           decayTable[i0].width=width;  
           for(j=1; allDecays(i,j,&pdg,&Len,decay,&width,&br) ;j++) if(br>0)
           { int k;  
             l=malloc(sizeof(txtListStr));
             l->txt=malloc(100); 
             l->next=L;
             sprintf(l->txt,"%E  %s -> ",br, pdg2name(pdg));
             sprintf(l->txt+strlen(l->txt),"%s",pdg2name(decay[0]));
             for(k=1;k<Len;k++) sprintf(l->txt+strlen(l->txt),", %s",pdg2name(decay[k]));
             L=l;
           }
           if(pdg0==pdg) 
           {  decayTable[i0].pdList[j0]=L; 
              if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname))
                           decayTable[i0].pdList[1-j0]=conBrList(L);
           } else 
           { decayTable[i0].pdList[1-j0]=L;
             if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname))
                           decayTable[i0].pdList[j0]=conBrList(L);
           }                
           if(LL) *LL=decayTable[i0].pdList[j0];
           return width;
        }
     }          
  }
  
  if(Q==NULL) for(i=0;i<nModelVars;i++) if(strcmp(varNames[i],"Q")==0){ Q= varValues+i; break;}
  if(Q) { Qstat=*Q; setQforParticle(name);}
    
  width=decay22List(name,&L);
  
  if(L) 
  {
    if(LL) *LL=L;
    decayTable[i0].pdList[j0]=L;
    if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname)) 
                 decayTable[i0].pdList[1-j0]=conBrList(L); 
    decayTable[i0].width=width;
    if(Q) {*Q=Qstat; calcMainFunc();}
    return width;
  }

  Lout=NULL;
  L= makeDecayList(name,3);
  massFilter(pMass(name),&L);
  gammaGluFilter(&L);
  if(L==NULL) 
  { L= makeDecayList(name,4);  
    massFilter(pMass(name),&L);
    gammaGluFilter(&L);
    nout=4;
  }  else nout=3;
      
  for(sum=0,l=L;l;l=l->next)  
  { numout* cc;
    int err;
    txtList newr;
    process2Lib(l->txt ,libName);
    cc=getMEcode(0,ForceUG,l->txt,NULL,"",libName);
    if(nout==3) width=width13(cc, 1, &err); else width=width14(cc, &err);
    if(width >0)
    {
      sum+=width;
      newr=malloc(sizeof(txtListStr));
      newr->next=Lout;
      Lout=newr;
      newr->txt=malloc(strlen(l->txt)+20);
      sprintf(newr->txt,"%E  %s",width,l->txt);
    }
  }
  cleanTxtList(L); 
  if(Lout)
  for(L=Lout;L;L=L->next)
  { char buff[100];
    sscanf(L->txt,"%lf %[^\n]",&width,buff);
    sprintf(L->txt,"%E %s",width/sum,buff);  
  }   
  if(LL) *LL=Lout;
  decayTable[i0].pdList[j0]=Lout;
  if(strcmp(ModelPrtcls[i0].name,ModelPrtcls[i0].aname)) 
               decayTable[i0].pdList[1-j0]=conBrList(Lout);
  decayTable[i0].width=sum;
  if(Q) { *Q=Qstat; calcMainFunc();}
  return sum;
}


double aWidth(char *name) { return pWidth(name,NULL);}

static int pListEq(char * txt1, char * txt2)  
{  char buff[100];
   char rd1[10][10];
   char rd2[10][10];
   int n1,n2,i1,i2;
   char *ch;
    
   strcpy(buff,txt1); while((ch=strchr(buff,','))) ch[0]=' ';
   
   n1=sscanf(buff,"%s %s %s %s %s %s %s %s %s %s",
   rd1[0],rd1[1],rd1[2],rd1[3],rd1[4],rd1[5],rd1[6],rd1[7],rd1[8],rd1[9]); 
   
   strcpy(buff,txt2); while((ch=strchr(buff,','))) ch[0]=' ';
   
   n2=sscanf(buff,"%s %s %s %s %s %s %s %s %s %s",
   rd2[0],rd2[1],rd2[2],rd2[3],rd2[4],rd2[5],rd2[6],rd2[7],rd2[8],rd2[9]); 
   
   if(n1!=n2) return 0;
   for(i1=0;i1<n1;i1++)
   { for(i2=0;i2<n2;i2++) if(strcmp(rd1[i1],rd2[i2])==0){rd2[i2][0]=0; break;}
     if(i2==n2) return 0;
   } 
   return 1;
}      

double findBr(txtList L, char * pattern)
{ char buff[100];
  char *ch;
  double width;
  
  for(;L;L=L->next)
  { 
     sscanf(L->txt,"%lf %[^\n]",&width,buff);
     ch=strstr(buff,"->");
     ch+=2;
     if( pListEq(ch,pattern)) return width;
  }
  return 0;   
}


